import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IDashboardComponent } from '../dashboard-component.model';
import { sampleWithRequiredData, sampleWithNewData, sampleWithPartialData, sampleWithFullData } from '../dashboard-component.test-samples';

import { DashboardComponentService } from './dashboard-component.service';

const requireRestSample: IDashboardComponent = {
  ...sampleWithRequiredData,
};

describe('DashboardComponent Service', () => {
  let service: DashboardComponentService;
  let httpMock: HttpTestingController;
  let expectedResult: IDashboardComponent | IDashboardComponent[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(DashboardComponentService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should create a DashboardComponent', () => {
      const dashboardComponent = { ...sampleWithNewData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.create(dashboardComponent).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a DashboardComponent', () => {
      const dashboardComponent = { ...sampleWithRequiredData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.update(dashboardComponent).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a DashboardComponent', () => {
      const patchObject = { ...sampleWithPartialData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of DashboardComponent', () => {
      const returnedFromService = { ...requireRestSample };

      const expected = { ...sampleWithRequiredData };

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toMatchObject([expected]);
    });

    it('should delete a DashboardComponent', () => {
      const expected = true;

      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult).toBe(expected);
    });

    describe('addDashboardComponentToCollectionIfMissing', () => {
      it('should add a DashboardComponent to an empty array', () => {
        const dashboardComponent: IDashboardComponent = sampleWithRequiredData;
        expectedResult = service.addDashboardComponentToCollectionIfMissing([], dashboardComponent);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(dashboardComponent);
      });

      it('should not add a DashboardComponent to an array that contains it', () => {
        const dashboardComponent: IDashboardComponent = sampleWithRequiredData;
        const dashboardComponentCollection: IDashboardComponent[] = [
          {
            ...dashboardComponent,
          },
          sampleWithPartialData,
        ];
        expectedResult = service.addDashboardComponentToCollectionIfMissing(dashboardComponentCollection, dashboardComponent);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a DashboardComponent to an array that doesn't contain it", () => {
        const dashboardComponent: IDashboardComponent = sampleWithRequiredData;
        const dashboardComponentCollection: IDashboardComponent[] = [sampleWithPartialData];
        expectedResult = service.addDashboardComponentToCollectionIfMissing(dashboardComponentCollection, dashboardComponent);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(dashboardComponent);
      });

      it('should add only unique DashboardComponent to an array', () => {
        const dashboardComponentArray: IDashboardComponent[] = [sampleWithRequiredData, sampleWithPartialData, sampleWithFullData];
        const dashboardComponentCollection: IDashboardComponent[] = [sampleWithRequiredData];
        expectedResult = service.addDashboardComponentToCollectionIfMissing(dashboardComponentCollection, ...dashboardComponentArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const dashboardComponent: IDashboardComponent = sampleWithRequiredData;
        const dashboardComponent2: IDashboardComponent = sampleWithPartialData;
        expectedResult = service.addDashboardComponentToCollectionIfMissing([], dashboardComponent, dashboardComponent2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(dashboardComponent);
        expect(expectedResult).toContain(dashboardComponent2);
      });

      it('should accept null and undefined values', () => {
        const dashboardComponent: IDashboardComponent = sampleWithRequiredData;
        expectedResult = service.addDashboardComponentToCollectionIfMissing([], null, dashboardComponent, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(dashboardComponent);
      });

      it('should return initial array if no DashboardComponent is added', () => {
        const dashboardComponentCollection: IDashboardComponent[] = [sampleWithRequiredData];
        expectedResult = service.addDashboardComponentToCollectionIfMissing(dashboardComponentCollection, undefined, null);
        expect(expectedResult).toEqual(dashboardComponentCollection);
      });
    });

    describe('compareDashboardComponent', () => {
      it('Should return true if both entities are null', () => {
        const entity1 = null;
        const entity2 = null;

        const compareResult = service.compareDashboardComponent(entity1, entity2);

        expect(compareResult).toEqual(true);
      });

      it('Should return false if one entity is null', () => {
        const entity1 = { id: 123 };
        const entity2 = null;

        const compareResult1 = service.compareDashboardComponent(entity1, entity2);
        const compareResult2 = service.compareDashboardComponent(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey differs', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 456 };

        const compareResult1 = service.compareDashboardComponent(entity1, entity2);
        const compareResult2 = service.compareDashboardComponent(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey matches', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 123 };

        const compareResult1 = service.compareDashboardComponent(entity1, entity2);
        const compareResult2 = service.compareDashboardComponent(entity2, entity1);

        expect(compareResult1).toEqual(true);
        expect(compareResult2).toEqual(true);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
