import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IPeriodVersion } from '../period-version.model';
import { sampleWithRequiredData, sampleWithNewData, sampleWithPartialData, sampleWithFullData } from '../period-version.test-samples';

import { PeriodVersionService, RestPeriodVersion } from './period-version.service';

const requireRestSample: RestPeriodVersion = {
  ...sampleWithRequiredData,
  creationDate: sampleWithRequiredData.creationDate?.toJSON(),
};

describe('PeriodVersion Service', () => {
  let service: PeriodVersionService;
  let httpMock: HttpTestingController;
  let expectedResult: IPeriodVersion | IPeriodVersion[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(PeriodVersionService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should create a PeriodVersion', () => {
      const periodVersion = { ...sampleWithNewData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.create(periodVersion).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a PeriodVersion', () => {
      const periodVersion = { ...sampleWithRequiredData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.update(periodVersion).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a PeriodVersion', () => {
      const patchObject = { ...sampleWithPartialData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of PeriodVersion', () => {
      const returnedFromService = { ...requireRestSample };

      const expected = { ...sampleWithRequiredData };

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toMatchObject([expected]);
    });

    it('should delete a PeriodVersion', () => {
      const expected = true;

      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult).toBe(expected);
    });

    describe('addPeriodVersionToCollectionIfMissing', () => {
      it('should add a PeriodVersion to an empty array', () => {
        const periodVersion: IPeriodVersion = sampleWithRequiredData;
        expectedResult = service.addPeriodVersionToCollectionIfMissing([], periodVersion);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(periodVersion);
      });

      it('should not add a PeriodVersion to an array that contains it', () => {
        const periodVersion: IPeriodVersion = sampleWithRequiredData;
        const periodVersionCollection: IPeriodVersion[] = [
          {
            ...periodVersion,
          },
          sampleWithPartialData,
        ];
        expectedResult = service.addPeriodVersionToCollectionIfMissing(periodVersionCollection, periodVersion);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a PeriodVersion to an array that doesn't contain it", () => {
        const periodVersion: IPeriodVersion = sampleWithRequiredData;
        const periodVersionCollection: IPeriodVersion[] = [sampleWithPartialData];
        expectedResult = service.addPeriodVersionToCollectionIfMissing(periodVersionCollection, periodVersion);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(periodVersion);
      });

      it('should add only unique PeriodVersion to an array', () => {
        const periodVersionArray: IPeriodVersion[] = [sampleWithRequiredData, sampleWithPartialData, sampleWithFullData];
        const periodVersionCollection: IPeriodVersion[] = [sampleWithRequiredData];
        expectedResult = service.addPeriodVersionToCollectionIfMissing(periodVersionCollection, ...periodVersionArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const periodVersion: IPeriodVersion = sampleWithRequiredData;
        const periodVersion2: IPeriodVersion = sampleWithPartialData;
        expectedResult = service.addPeriodVersionToCollectionIfMissing([], periodVersion, periodVersion2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(periodVersion);
        expect(expectedResult).toContain(periodVersion2);
      });

      it('should accept null and undefined values', () => {
        const periodVersion: IPeriodVersion = sampleWithRequiredData;
        expectedResult = service.addPeriodVersionToCollectionIfMissing([], null, periodVersion, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(periodVersion);
      });

      it('should return initial array if no PeriodVersion is added', () => {
        const periodVersionCollection: IPeriodVersion[] = [sampleWithRequiredData];
        expectedResult = service.addPeriodVersionToCollectionIfMissing(periodVersionCollection, undefined, null);
        expect(expectedResult).toEqual(periodVersionCollection);
      });
    });

    describe('comparePeriodVersion', () => {
      it('Should return true if both entities are null', () => {
        const entity1 = null;
        const entity2 = null;

        const compareResult = service.comparePeriodVersion(entity1, entity2);

        expect(compareResult).toEqual(true);
      });

      it('Should return false if one entity is null', () => {
        const entity1 = { id: 123 };
        const entity2 = null;

        const compareResult1 = service.comparePeriodVersion(entity1, entity2);
        const compareResult2 = service.comparePeriodVersion(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey differs', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 456 };

        const compareResult1 = service.comparePeriodVersion(entity1, entity2);
        const compareResult2 = service.comparePeriodVersion(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey matches', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 123 };

        const compareResult1 = service.comparePeriodVersion(entity1, entity2);
        const compareResult2 = service.comparePeriodVersion(entity2, entity1);

        expect(compareResult1).toEqual(true);
        expect(compareResult2).toEqual(true);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
